#include <iostream>
#include <math.h>
#include "GLWindow.h"
#include "readpng.h"

using namespace std;


void GLWindow::Init() {
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glClearColor( .8, .8, .8, 0 );
   glClearDepth( 1 );
   scale = 1;
   xTranslate = 0;
   yTranslate = 0;
   zooming = false;
   translating = false;
   movingPolygon = false;
   creatingPolygon = false;
}

void GLWindow::Key(unsigned char key, int x, int y) {
   switch (key) {
      case 'w':
         grid.drawStyle = Grid::DRAW_WIRE;
         break;
      case 't':
         grid.drawStyle = Grid::DRAW_TEXTURE;
         break;
      case 'c':
         grid.ClearPlygon();
         break;
      case 'r':
         grid.ResetPolygon();
         break;
      case 'q':
	exit(0);
	
   }
   Draw();
}


void GLWindow::Mouse( int button, int state, int x, int y ) {
   if (state != GLUT_DOWN) {
      translating = false;
      zooming = false;
      movingPolygon = false;
      creatingPolygon = false;
      oldX = oldY = -1;
      return;
   }
   if(glutGetModifiers() & GLUT_ACTIVE_CTRL) { 
      if (button == GLUT_LEFT_BUTTON) {
         movingPolygon = true;
      }
      if (button == GLUT_RIGHT_BUTTON) {
         creatingPolygon = true;
      }
   }
   if(glutGetModifiers() & GLUT_ACTIVE_SHIFT) { 
      if (button == GLUT_LEFT_BUTTON) {
         translating = true;
      }
      if (button == GLUT_RIGHT_BUTTON) {
         zooming = true;
      }
   }
   float myX, myY;
   myX =  (float)(x*2.0 - width)/(float)scale - xTranslate;
   myY = -(float)(y*2.0 - height)/(float)scale - yTranslate;
   if (creatingPolygon) {
      grid.AddPolygonPoint(myX, myY);
      Draw();
   }
   oldX = myX;
   oldY = myY;
}

void GLWindow::MouseMotion( int x, int y ) {
   float myX, myY;
   myX =  (float)(x*2.0 - width)/(float)scale - xTranslate;
   myY = -(float)(y*2.0 - height)/(float)scale - yTranslate;
   if (oldX >= 0 && oldY >= 0) {
      if (movingPolygon) {
         grid.MovePolygonPoint(oldX, oldY, (myX - oldX), (myY-oldY));
         Draw();
      }
      else if (translating) {
         xTranslate += (myX - oldX);
         yTranslate += (myY - oldY);
         glutPostRedisplay();
      }
      else if (zooming) {
         if ((myY-oldY) > 0)
            scale *= 1.05;
         else 
            scale /= 1.05;
         glutPostRedisplay();
      }
   }
   oldX = myX;
   oldY = myY;
}

void GLWindow::Resize(int _width, int _height) {
   if( _width <= 0 || _height <= 0 ) return;
   width = _width;
   height = _height;
   glViewport(0,0, _width, _height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   int vPort[4];
   glGetIntegerv(GL_VIEWPORT, vPort);
   glOrtho(-vPort[2], vPort[2], -vPort[3], vPort[3], -1000, 1000);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   scale = _height*1.75;
   xTranslate = -.5;
   yTranslate = -.5;
}

void GLWindow::Draw() {
   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
   glLoadIdentity();

   glScalef(scale, scale, 1);
   glTranslatef(xTranslate, yTranslate, 0);

   glLineWidth(1);
   glColor3f(.3f, .3f, .3f);
   glBegin(GL_LINE_LOOP);
      glVertex2f (.0f,  .0f);
      glVertex2f (.0f, 1.0f);
      glVertex2f(1.0f, 1.0f);
      glVertex2f(1.0f, 0.0f);
   glEnd();
   grid.Draw();
   glFlush();
   glutSwapBuffers();
}



bool GLWindow::LoadTexture(char * filename) {

  GLuint texNum;

  int width, height;

  cerr << "Attempting to read texture: " << filename << endl;
  png_bytepp texData = readpng(filename, &width, &height);
  if (texData == NULL) cerr << "Failed to read texture." << endl;
  char *tmp = (char*)malloc(width*height*4);

  for(int y=0; y < height; y++)
    memcpy(tmp+width*y*4, texData[height-y-1], width*4);

  glGenTextures(1, &texNum);
  glBindTexture(GL_TEXTURE_2D, texNum);

  GLfloat borderColor[] = {1.0f, 1.0f, 1.0f, 1.0f};
  glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, borderColor);
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height,
	       0, GL_RGBA, GL_UNSIGNED_BYTE, tmp);

  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

  free(texData);
  free(tmp);

  cerr << "Finished loading texture." << endl;

  grid.drawStyle = Grid::DRAW_TEXTURE;
  return true;
}


