#include <stdlib.h>
#include <GL/glut.h>
#include "Grid.h"

using namespace std;

Grid::Grid()
{
   points = NULL;
   size = 0;
}

void Grid::AddPolygonPoint(const float & x, const float & y) {
   if (x >= 0 && x <= 1 && y >= 0 && y <= 1) {
      polygon.push_back(Vector2(x, y));
      movedPolygon.push_back(Vector2(x, y));
   }
}

void Grid::MovePolygonPoint(const float & x, const float & y, const float & dx, const float & dy) {
   if (movedPolygon.size() < 0) return;
   int closest = 0;
   Vector2 v(x, y);
   float dist = (movedPolygon[0] - v).dotItself();
   for (unsigned int i = 1; i < movedPolygon.size(); ++i) {
      if ((movedPolygon[i] - v).dotItself() < dist) {
         closest = i;
         dist = (movedPolygon[i] - v).dotItself();
      }
   }
   movedPolygon[closest].x() += dx;
   movedPolygon[closest].y() += dy;
}

void Grid::Resize(int n)
{
   if (points) {
      for (int i = 0; i < size; ++i)
         delete [] points[i];
      delete [] points;
   }

   size = n;
   points = new Vector2 *[n];
   for (int i = 0; i < size; ++i)
      points[i] = new Vector2[size];

   float dx = 1.0/(float)(size-1);
   for (int i = 0; i < size; ++i) {
      for (int j = 0; j < size; ++j) {
         points[i][j].x() = i*dx;
         points[i][j].y() = j*dx;
      }
   }
   drawStyle = DRAW_WIRE;
}

void Grid::Draw() {
   glLineWidth(.5f);

   if (drawStyle == DRAW_WIRE) {
      glDisable(GL_TEXTURE_2D);
      glColor3f(0.1, 0.4, 0.1);
      for (int i = 0; i < size-1; ++i) {
         for (int j = 0; j < size-1; ++j) {
            glBegin(GL_LINE_LOOP);
            glVertex2f(points[i][j].x(), points[i][j].y());
            glVertex2f(points[i+1][j].x(), points[i+1][j].y());
            glVertex2f(points[i][j+1].x(), points[i][j+1].y());
            glEnd();
            glBegin(GL_LINE_LOOP);
            glVertex2f(points[i+1][j].x(), points[i+1][j].y());
            glVertex2f(points[i+1][j+1].x(), points[i+1][j+1].y());
            glVertex2f(points[i][j+1].x(), points[i][j+1].y());
            glEnd();
         }
      }
   }
   else {
      glColor3f(1, 1, 1);
      glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
      glEnable(GL_TEXTURE_2D);
      glMatrixMode(GL_TEXTURE);
      glLoadIdentity();	
      glMatrixMode(GL_MODELVIEW);

      for (int i = 0; i < size-1; ++i) {
         for (int j = 0; j < size-1; ++j) {
            glBegin(GL_POLYGON);
            glTexCoord2f(points[i][j].x(), points[i][j].y());
            glVertex2f  (points[i][j].x(), points[i][j].y());
            glTexCoord2f(points[i+1][j].x(), points[i+1][j].y());
            glVertex2f  (points[i+1][j].x(), points[i+1][j].y());
            glTexCoord2f(points[i][j+1].x(), points[i][j+1].y());
            glVertex2f  (points[i][j+1].x(), points[i][j+1].y());
            glEnd();
            glBegin(GL_POLYGON);
            glTexCoord2f(points[i+1][j].x(), points[i+1][j].y());
            glVertex2f  (points[i+1][j].x(), points[i+1][j].y());
            glTexCoord2f(points[i+1][j+1].x(), points[i+1][j+1].y());
            glVertex2f  (points[i+1][j+1].x(), points[i+1][j+1].y());
            glTexCoord2f(points[i][j+1].x(), points[i][j+1].y());
            glVertex2f  (points[i][j+1].x(), points[i][j+1].y());
            glEnd();
         }
      }
   }
   glDisable(GL_TEXTURE_2D);
   glLineWidth(1.7);
   glPointSize(4.2);
   glColor3f(.7f, .7f, .8f);
   glBegin(GL_LINE_LOOP);
   for (unsigned int i = 0; i < polygon.size(); ++i) {
      glVertex2f(polygon[i].x(), polygon[i].y());
   }
   glEnd();
   glColor3f(.25f, .25f, .65f);
   glBegin(GL_POINTS);
   for (unsigned int i = 0; i < polygon.size(); ++i) {
      glVertex2f(polygon[i].x(), polygon[i].y());
   }
   glEnd();
   glColor3f(.8f, .7f, .8f);
   glBegin(GL_LINE_LOOP);
   for (unsigned int i = 0; i < polygon.size(); ++i) {
      glVertex2f(movedPolygon[i].x(), movedPolygon[i].y());
   }
   glEnd();
   glColor3f(.85f, .25f, .65f);
   glBegin(GL_POINTS);
   for (unsigned int i = 0; i < polygon.size(); ++i) {
      glVertex2f(movedPolygon[i].x(), movedPolygon[i].y());
   }
   glEnd();
}


Grid::~Grid(void)
{
   if (!points) return;

   for (int i = 0; i < size; ++i)
      delete [] points[i];
   delete [] points;
}
